# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2008
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Outil de gestion des configurations dansguardian
#   édite les fichiers locaux de configuration utilisateur
#
###########################################################################

"""
    Gestion des configurations de DansGuardian.
"""
import re
from os.path import join
from pyeole.service import service_out
from amon.dansguardian import config
from amon.dansguardian.config_dans import desactive, active_c, active_m
#from amon.backend import get_filter_zones, is_auth

columns_list = range(0, len(config.optional_policies))
columns_libelle = ['Défaut'] + [str(x) for x in range(1, len(config.optional_policies))]
categ_re = re.compile(r"(?P<commented>#?)(?P<include>.Include</var/lib/blacklists/eole/(?P<categ>[^/]+)/.*)")

#association mot-clé<->nom de fichier
kw_to_files_dict = {
        'blacklist':   join(config.DANS_LOCAL_SUBDIR, 'domains'),
        'whitelist':   join(config.DANS_LOCAL_SUBDIR, 'whites'),
        'extension':  join(config.DANS_LOCAL_SUBDIR, 'extensions'),
        'mime':        join(config.DANS_LOCAL_SUBDIR, 'types_mime'),
        "filtres_opt": [join(config.DANS_LOCAL_SUBDIR, "bannedsitelist"),
                        join(config.DANS_LOCAL_SUBDIR, "bannedurllist")],
        "greysitelist": join(config.DANS_LOCAL_SUBDIR, "site_liste_blanche")
            }

def restart():
    """ relance dansguardian """
    ret = service_out('dansguardian', 'restart', container='proxy')
    if ret[0] != 0:
        return False
    return True

def reload():
    """ reload dansguardian """
    ret = service_out('dansguardian', 'reload', container='proxy')
    if ret[0] != 0:
        return False
    return True

def get_groupslist(zone, _type='user'):
    """
        renvoie ipgroupslist ou filtergroupslist
        selon le type demandé
    """
    if _type == 'user': # is_auth(zone):
        return config.filtergroupslist % zone
    else:
        return config.ipgroupslist % zone

# Modification de la configuration d'une zone
def set_policy(zone, policy_index, item, _type='user'):
    """Ajoute l'item (utilisateur ou groupe de machines) de type _type à la politique de
       filtrage optionnelle désignée par policy_index, pour la zone zone.
       @param : zone = 0/1
       @policy_index : 0/1/2/3/4/5/6/7/8
       @item : nom ou plage d'ip
       @_type : user/ip
    """
    filename = get_groupslist(zone, _type)
    conf_datas = file(filename).read()
    if item in conf_datas:
        conf_datas = conf_datas.splitlines()
        for index, line in enumerate(conf_datas):
            if item in line:
                conf_datas[index] = "%s=filter%d" % (item, policy_index)
                break
        to_w = "\n".join(conf_datas)
        to_w += "\n"
        f = file(filename, 'w')
        f.write(to_w)
        f.close()
    else:
        f = file(filename, 'a')
        f.write("%s=filter%d\n" % (item, policy_index))
        f.close()

def unset_policy(zone, item, _type='user'):
    """
        Supprime l'item (utilisateur ou groupe de machines) de type _type
        de la liste des items associés à des politiques.
    """
    #filename = config.filtergroupslist%zone
    filename = get_groupslist(zone, _type)
    conf_datas = file(filename).read()
    if item in conf_datas:
        conf_datas = conf_datas.splitlines()
        for index, line in enumerate(conf_datas):
            if item in line:
                conf_datas.remove(line)
                break
        to_w = "\n".join(conf_datas)
        to_w += '\n'
        f = file(filename, 'w')
        f.write(to_w)
        f.close()

def get_policies(zone, _type='user'):
    """ renvoie le contenu de config.filtergroupslist ou ipgroupslist formaté """
    #filename = config.filtergroupslist%zone
    filename = get_groupslist(zone, _type=_type)
    conf_datas = file(filename).read().splitlines()
    policies = {}
    for policy in config.policies:
        policies[policy] = []
    for line in conf_datas:
        line = line.split('=')
        for policy in config.policies:
            _filter = 'filter%d' % policy
            if len(line)>1:
                if _filter == line[1].strip():
                    policies[policy].append(line[0].strip())
                    break
    return policies

def get_policy(zone, item, _type='user'):
    """ renvoie la police appliquée à l'item item pour la zone zone """
    for policy, items in get_policies(zone, _type=_type).items():
        if item in items:
            return policy
    return None

def get_specials(zone, keyword=None):
    """ renvoie la liste des items (user et ip) spéciaux (filtre 2,3,4)
        keyword: 2 pour modérateur, 3 pour user interdits, 4 pour les users listes blanches
    """
    if keyword is None:
        return dict([(policy, items)for policy, items in get_policies(zone).items()if policy in config.spec_policies])
    else:
        return dict([(policy, items)for policy, items in get_policies(zone).items()if policy == keyword])


        ########### modele de filtrage
def set_filtrage(zone, mode='on'):
    """ met en place le modèle de filtrage (aucun, meta ou toute la page)
        pour la zone zone
        mode = 'meta':activation du filtrage par méta
               'on'  :activation du filtrage de la page complète
               sinon désactivation du filtrage de contenu
    """
    # petit hack pour permettre l'appel avec les valeurs numeriques du fichier sauvegardant le status
    filter_keys = {'1':'meta', '2':'on', '0':'default'}
    if mode in filter_keys.keys():
        mode = filter_keys[mode]

    if mode == 'meta':
        active_m(zone)
    elif mode == 'on':
        active_c(zone)
    else:
        desactive(zone)
# Modification des configurations personnalisée par politique

def kw_to_files(keyword, zone, indice):
    """ renvoie le nom des fichiers """
    filename = kw_to_files_dict[keyword]
    if type(filename) == list:
        return [f % (zone, indice) for f in filename]
    return filename % (zone, indice)

def handle_policy_files(function, keyword, zone, collection):
    for index, indice in enumerate(config.optional_policies):
        filename = kw_to_files(keyword, zone, indice)
        function(collection, filename, index)
    return collection

def load_site_list(keyword='blacklist', zone=''):
    """ charge la liste des sites interdits/autorisés, les listes d'extensions
        et de type mime interdits de la zone zone pour les politiques de
        filtrage.
    """
    def add_index_to_sites_from_file(sites, filename, index=None):
        f = open(filename)
        for line in f.xreadlines():
            line = line.strip()
            if line in sites:
                sites[line][index] = 1
            else:
                new = [0]*len(config.optional_policies)
                new[index] = 1
                sites[line] = new
        f.close()
    return handle_policy_files(add_index_to_sites_from_file, keyword,
            zone, {})

def save_site_list(sites, keyword="blacklist", zone=""):
    """ Écrit la liste des sites interdits/autorisés de la zone zone
        pour les politiques de filtrage.
    """
    def write_sites_to_file(files, filename, i):
        print "ecriture de %s dans %s" % (files[i], filename)
        f = open(filename, "w")
        for site in files[i]:
            f.write(site + "\n")
        f.close()

    # On range d'abord les listes par fichier dans files :
    files = []
    for i in config.optional_policies:
        files.append([])
    for site, booleans in sites.items():
        for i, b in enumerate(booleans):
            if b:
                files[i].append(site)

    # On peut maintenant écrire la liste dans chaque fichier.
    handle_policy_files(write_sites_to_file, keyword, zone, files)

def load_category_list(keyword="filtres_opt", zone=""):
    """Charge une liste de catégories de la zone zone."""
    def read_status_of_categories(categs, filename, index):
        if type(filename) == list:
            filename = filename[0]
        f = open(filename)
        for line in f.xreadlines():
            m = categ_re.search(line)
            if not m or len(m.groups()) < 2:
                continue
            enabled = m.group("commented") == ""
            categ = m.group("categ")
            if not categ in categs:
                # New categ, all enabled by default:
                categs[categ] = [1]*len(config.optional_policies)
            if not enabled:  # Only disable (since categories are enabled
                             # by default):
                categs[categ][index] = 0
        f.close()
        return categs
    return handle_policy_files(read_status_of_categories, keyword, zone, {})

def save_category_list(categs, keyword="filtres_opt", zone=""):
    """Sauve une liste de catégories de la zone zone."""
    def write_status_of_categories(categs, filenames, index):
        if type(filenames) != list:
            filenames = list(filenames)
        for filename in filenames:
            f = open(filename)
            lines_to_write = []  # Lignes à écrire plus tard dans le fichier
                                 # qu'on écrasera.
            for line in f.xreadlines():
                m = categ_re.search(line)
                if not m or len(m.groups()) < 2:
                    continue
                if categs[m.group("categ")][index]:
                    lines_to_write.append(m.group("include"))
                else:
                    lines_to_write.append("#" + m.group("include"))
            f.close()
            f = open(filename, "w")
            for line in lines_to_write:
                f.write(line + "\n")
            f.close()
        return categs
    return handle_policy_files(write_status_of_categories, keyword, zone,
                               categs)

def option_and_index_to_name(option, index):
    """ forme un name à partir d'un nom d'option et d'un index (entier) """
    return "%s_%s" % (option, index)

def name_to_option_and_index(name):
    """ Récupère option et index (l'inverse d'option_and_index_to_name) """
    where = name[::-1].find("_")
    if where == -1:
        raise ValueError("Aucun indice de colonne trouvé dans le nom \"%s\""
                % name)
    try:
        return name[:-where-1], int(name[len(name)-where:])
    except ValueError:  # La partie après le séparateur n'est pas un entier.
        raise ValueError("Aucun indice de colonne trouvé dans le nom \"%s\""
                % name)

def load_greysitelist(zone):
    """ renvoie la liste blanche de sites pour la zone zone """
    return file(kw_to_files('greysitelist', zone, 4)).read().splitlines()

def save_greysitelist(zone, sitelist):
    """ écrit une liste de site interdits """
    sitelist_f = file(kw_to_files('greysitelist', zone, 4), 'w')
    sitelist_f.write('\n'.join(sitelist))
    sitelist_f.close()
