# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2008
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
###########################################################################
"""
    Met en place la configuration dansguardian

# definition d'un schema de creation de config:
# dans /etc/
# pour zone de 1 à n:
#       créer /etc/dansguardianZ/
#       créer /etc/dansguardianZ/dansguardian.conf(zone)
#       créer les /etc/dansguardianZ/downloadmanager/
#       pour indice de 1 à num_politique:
#             créer /etc/dansguardianZ/dansguardianfi.conf(zone, indice)
#             créer /etc/dansguardianZ/lists/fichiers(zone, indice) depuis les templates

# dans /var/lib/blacklists/
# créer les répertoires communs à toutes les zones
# pour zone de 1 à n:
#     créer /var/lib/blacklists/<zone>/
#     créer un répertoire commun aux différentes politique d'une zone
#     pour indice de 1 à num_politique:
#           créer un répertoire /var/lib/blacklists/<zone>/<indice>/
#           créer les fichiers locaux (domain, white, extensions, typesmime)

## Les politiques listes blanches, modérateurs et utilisateurs interdits:
# dans /etc:
# pour zone de 1 à n:
#       créer les fichiers dansguardianf 2,3,4 depuis les templates associés
#       créer les répertoires f2, f3, f4
#       créer les fichiers depuis les templates réutilisation de la fonction utilisée
#                    pour les autres groupes
"""
from os import mkdir, remove
from os.path import join, isdir, isfile
import commands

from amon.dansguardian.config import *
from amon.backend import ifaces_filename
from amon.dansguardian.dans_tools import set_filtrage

debug = False

def get_plugin(active_squid, num_zone):
    """ renvoie les lignes pour configurer le plugin d'auth
        dans dansguardian.conf cette interface
    """
    if active_squid == "non":
        plugins = "authplugin = '%s'" % (join(get_instance_dir(num_zone),
                                              'common', 'ip.conf'))
    else:
        plugins = """authplugin = '%s'
authplugin = '%s'
authplugin = '%s'""" % (join(get_instance_dir(num_zone), 'authplugins', 'proxy-ntlm.conf'),
                        join(get_instance_dir(num_zone), 'authplugins', 'proxy-basic.conf'),
                        join(get_instance_dir(num_zone), 'common', 'ip.conf')
                       )
    return plugins

def get_ip_conf(ips):
    """ renvoie les lignes du fichier de configuration avec les adresse ip """
    lines = ''
    for ip in ips:
        lines += 'filterip=%s\n' % ip
    return lines

def get_instance_dir(num_zone):
    """
    renvoie le chemin vers le répertoire de conf
    de l'instance numéro num_zone
    """
    return DANS_FULL_DIR % num_zone

def make_instance_dir(num_zone):
    """ crée le répertoire dans /etc/ pour le dansguardian numéro num_zone
        purge les fichiers temporaires (utilisés pour mémoriser des configs)
    """
    if isfile(ifaces_filename):
        remove(ifaces_filename)
    if not isdir(get_instance_dir(num_zone)):
        if debug:
            print "Création de %s" % get_instance_dir(num_zone)
        mkdir(get_instance_dir(num_zone))

def make_instance_conf(num_zone, ips, active_squid):
    """
        crée le fichier dansguardian.conf pour la zone num_zone
    """
    dans_filename = 'dansguardian.conf'
    if num_zone == 0 or num_zone == "0":
        pre_dans_filename = 'dansguardian1.conf'
    elif num_zone == 1 or num_zone == "1":
        pre_dans_filename = 'dansguardian2.conf'
    else:
        pre_dans_filename = 'dansguardian3.conf'
    # ouverture du template:
    template = file(join(EOLE_DIR, pre_dans_filename)).read()

    template = template.replace('$z', str(num_zone))
    # choix du plugin d'auth
    template = template.replace("$p", get_plugin(
                                        active_squid,
                                        num_zone)
                               )

    # configuration de ou des ips
    template = template.replace("$ip", get_ip_conf(ips))

    # écriture du fichier
    if debug:
        print "Écriture de %s" % join(get_instance_dir(num_zone), dans_filename)
    dans_file = file(join(get_instance_dir(num_zone), dans_filename), 'w')
    dans_file.write(template)
    dans_file.close()

def make_spec_policy_conf(num_zone, indice):
    """
    crée les fichiers de politiques fixes
    (liste blanche/modérateur/users interdits)
    """
    dansfx_filename = 'dansguardianf%d.conf' % indice

    if debug:
        print "Création de %s" % join(get_instance_dir(num_zone),
                                      dansfx_filename)
    # ouverture du template:
    template = file(join(EOLE_DIR, dansfx_filename)).read()
    template = template.replace('$z', str(num_zone))
    template = template.replace('$s', str(indice))
    dansfx_file = file(join(get_instance_dir(num_zone), dansfx_filename), 'w')
    dansfx_file.write(template)
    dansfx_file.close()


def make_policy_conf(num_zone, indice):
    """ crée le fichier /etc/dansguardian<num_zone>/dansguardianf<indice>.conf
        pour la politique numéro indice de la zone num_zone
    """
    dansfx_filename = 'dansguardianf%d.conf' % indice
    template_filename = 'dansguardianfX.conf'

    if debug:
        print "Création de %s" % join(get_instance_dir(num_zone),
                                      dansfx_filename)
    # ouverture du template:
    template = file(join(EOLE_DIR, template_filename)).read()
    template = template.replace('$z', str(num_zone))
    template = template.replace('$s', str(indice))
    dansfx_file = file(join(get_instance_dir(num_zone), dansfx_filename), 'w')
    dansfx_file.write(template)
    dansfx_file.close()

def make_instance_common_dir(num_zone):
    """ crée le repertoire /etc/dansguardian<zone>/common/
        et place les fichiers importés depuis dansguardian.conf:
        filtergroupslist, exceptioniplist et exceptionsitelist
    """
    dirname = join(get_instance_dir(num_zone),'common')
    if not isdir(dirname):
        mkdir(dirname)
    for template_name in instance_common_templates:
        template = file(join(COMMON_TEMPLATE_DIR, template_name)).read()
        template = template.replace('$z', str(num_zone))
        f_name = join(dirname, template_name)
        fic = file(f_name, 'w')
        fic.write(template)
        fic.close()

def copydata(dirname, dest):
    """ utilitaire de copie de répertoire """
    commands.getstatusoutput("cp -R %s %s" % (dirname, dest))

def make_instance_fix_subdirs(num_zone):
    """
        copie downloadmanagers et authplugins dans /etc/dansguardian<zone>/
    """
    for dirname in instance_fix_subdirs:
        if debug:
            print "Copie de %s" % dirname
        copydata(join(EOLE_DIR, dirname),
                join(get_instance_dir(num_zone), dirname))

def make_clamd_spec_dirs(num_zone):
    """
        Crée le répertoire contentscanners, templatise les fichiers
        depuis /usr/share/eole/dansguardian/contentscanners
        et les copies dans le répertoires
    """
    dirname = join(get_instance_dir(num_zone), 'contentscanners')
    if not isdir(dirname):
        mkdir(dirname)
    for template_name in instance_clamd_templates:
        template = file(join(CLAMD_TEMPLATE_DIR, template_name)).read()
        template = template.replace('$z', str(num_zone))
        f_name = join(dirname, template_name)
        fic = file(f_name, 'w')
        fic.write(template)
        fic.close()

def make_instance_spec_subdirs(num_zone, indice):
    """
        copie les sous-répertoires lists de /usr/share/eole/dansguardian/
        dans /etc/dansguardian<zone>/listsf<indice>

    """
    for dirname in instance_spec_subdirs:
        dest_name = "%sf%d" % (dirname, indice)
        if debug:
            print "Copie de %s vers %s" % (dirname, dest_name)
        copydata(join(EOLE_DIR, dirname),
                join(get_instance_dir(num_zone), dest_name))

def make_instance_subdirs_template(num_zone, indice):
    """
        crée les fichiers dans /etc/dansguardian<zone>/listsf<indice>/
        pour la politique indice de la zone num_zone
    """
    for template_name in instance_subdirs_templates:
        template = file(join(TEMPLATE_DIR, template_name)).read()
        template = template.replace('$z', str(num_zone))
        template = template.replace('$s', str(indice))
        conf_f_name = template_name
        if debug:
            print "listsf%d/Templatisation de %s" % (indice, conf_f_name)
        conf_f = file(join(get_instance_dir(num_zone),
                           'listsf%d' % indice,
                           conf_f_name), 'w')
        conf_f.write(template)
        conf_f.close()

def make_whitelist_subdirs_template(num_zone, indice):
    """ crée les fichiers spécifiques à la liste blanche utilisateur
        dans /etc/dansguardian<zone>/listsf4/
    """
    for template_name in instance_whitelist_templates:
        template = file(join(WHITELIST_TEMPLATE_DIR, template_name)).read()
        template = template.replace('$z', str(num_zone))
        template = template.replace('$s', str(indice))
        conf_f_name = template_name
        if debug:
            print "listsf%d/Templatisation de %s" % (indice, conf_f_name)
        conf_f = file(join(get_instance_dir(num_zone),
                           'listsf%d' % indice,
                           conf_f_name), 'w')
        conf_f.write(template)
        conf_f.close()

def create_main_dansguardian_conf(ifaces):
    """ entrée pour la création des répertoires et fichiers
        pour la configuration dansguardian
    """
# pour zone de 1 à n:
#       créer /etc/dansguardian<zone>/
#       créer /etc/dansguardian<zone>/dansguardian.conf(zone)
#       créer les /etc/dansguardianZ/downloadmanager/
#       pour indice de 1 à num_politique:
#             créer /etc/dansguardian<zone>/dansguardianfi.conf
#             créer /etc/dansguardian<zone>/listsf<indice/fichiers
#                           depuis les templates pour les politiques 1,4,5,6,7
#             écraser les fichiers différents dans /etc/dansguardian<zone>/listsf<indice/
#                           depuis template pour la liste blanche
    def get_active_squid(configuration):
        """
            renvoie l'activation de l'authentification squid sur une interface
            renvoie oui si il n'y a que du oui et non si il y a un seul non
            car si il y a un oui et un non,
            il ne faut pas activer les plugins d'auth (cf bug https)
        """
        active = "oui"
        for iface in configuration:
            if iface['activer_squid_auth'] == 'non':
                return 'non'
            else:
                pass
        return active

    def is_there_auth(configuration):
        """
            y-a-t-il une interface authentifiée dans la zone
        """
        for iface in configuration:
            if iface['activer_squid_auth'] == 'oui':
                return True
        return False

    active_ead_actions()
    for num_zone, value in ifaces.items():
        if value != []:
            ips = [iface["adresse_ip"] for iface in value]
            for iface in value:
                ips.extend(iface["adresse_ip_vlans"])
            active_squid = get_active_squid(value)
#            active_squid = value[0]['activer_squid_auth']

            make_instance_dir(num_zone)
            make_instance_conf(num_zone, ips, active_squid)
            make_instance_common_dir(num_zone)
            make_instance_fix_subdirs(num_zone)
            make_clamd_spec_dirs(num_zone)
            for policy in range(1, num_policies+1): #FIXME: nombre de politique à configurer
                if policy in spec_policies:
                    make_spec_policy_conf(num_zone, policy)
                else:
                    make_policy_conf(num_zone, policy)
                # les utilisateurs interdits et modérateurs pointent
                # vers les fichiers de la politique par défaut (1)
                if policy not in other_spec_policies:
                    make_instance_spec_subdirs(num_zone, policy)
                    make_instance_subdirs_template(num_zone, policy)
                if policy in whitelist_policy:
                    # ecrasement des fichiers particulier à la liste blanche (template)
                    make_whitelist_subdirs_template(num_zone, policy)
            if not is_there_auth(value):
                desactive_ead_user_action(num_zone)
            if isfile(dans_status%num_zone):
                status = file(dans_status % num_zone, 'r').read().splitlines()[0]
                if status in ["0", "1", "2"]:
                    set_filtrage(num_zone, status)
                else:
                    remove(dans_status % num_zone)
        else:
            desactive_ead_action(num_zone) #desactivation de la zone num_zone dans l'ead2

def desactive_ead_action(num_zone):
    """
        désactive les lignes pour la configuration
        de la politique num_zone
    """
    for perms in [amon_perms, proxy_perms]:
        if isfile(perms):
            try:
                lines = file(perms).read().splitlines()
            except:
                print "Impossible de lire le fichier %s" % perms
                continue
            for indice, line in enumerate(lines):
                # desactivation des zones non utilisées
                if '=' in line:
                    substring = '_%s' % NAME_DICT[num_zone]
                    rolename = 'admin_pedago'
                    action_name = line.split('=')[0]
                    if action_name != rolename and substring in action_name:
                        lines[indice] = '#%s' % line
            perm_f = file(perms,'w')
            perm_f.write('\n'.join(lines))
            perm_f.close()

def desactive_ead_user_action(num_zone):
    """
        désactive la gestion utilisateur dans le cas où l'authentification
        n'est pas active sur la zone
    """
    user_actions_name = ("user",)# "navigation_moderateur","navigation_banned_user",
                                 # "navigation_whitesitelist","navigation_whitelist")
    perms = proxy_perms
    try:
        lines = file(perms).read().splitlines()
    except:
        print "Impossible de lire le fichier %s" % perms
        return
    for indice, line in enumerate(lines):
        # desactivation des zones non utilisées
        for action in user_actions_name:
            act_name = '%s_%s' % (action, NAME_DICT[num_zone])
            if line.startswith(act_name):
                lines[indice] = '#%s' % line
    perm_f = file(perms,'w')
    perm_f.write('\n'.join(lines))
    perm_f.close()

def active_ead_actions():
    """ active toutes les actions ead pour chaque zone """
    for perms in [amon_perms, proxy_perms]:
        if isfile(perms):
            try:
                lines = file(perms).read().splitlines()
            except:
                print "Impossible de lire le fichier %s" % perms
                continue
            for indice, line in enumerate(lines):
                # activation des actions pour toutes les zones
                for zone in NAME_DICT.values():
                    if '_%s' % zone in line:
                        lines[indice] = '%s' % line.replace('#', '')
            perm_f = file(perms,'w')
            perm_f.write('\n'.join(lines))
            perm_f.close()

"""
*****    FICHIERS ET REPERTOIRES LOCAUX    *****
Création-only des fichiers de configurations personnalisables,
Dans un fonctionnement normale :
    - création seulement à l'instance
    - vérification quand même au reconfigure
"""

def get_local_instance_dir(num_zone):
    """
        renvoie le chemin vers la conf locale pour la zone num_zone
    """
    return DANS_LOCAL_DIR % num_zone

def make_local_instance_dir(num_zone):
    """
        crée les repertoires /var/lib/blacklists/dansguardian<zone>/
        et les répertoires /var/lib/blacklists/dansguardian<zone>/common/
    """
    if not isdir(get_local_instance_dir(num_zone)):
        mkdir(get_local_instance_dir(num_zone))

    if not isdir(join(get_local_instance_dir(num_zone), 'common')):
        mkdir(join(get_local_instance_dir(num_zone), 'common'))

def make_local_instance_subdirs(num_zone, indice):
    if not isdir(DANS_LOCAL_SUBDIR%(num_zone, indice)):
        mkdir(DANS_LOCAL_SUBDIR%(num_zone, indice))

def make_local_instance_common_conf_files(num_zone):
    """
        crée les fichiers 'spusers', 'users', 'hosts', 'filtergroupslist', 'ipgroupslist', 'urls'
        dans /var/lib/blacklists/dansguardian<zone>/common/
    """
    # les fichiers locaux spécifiques à une seule politique de filtrage
    for f_name in local_common_conf_files:
        f_filename = join(DANS_LOCAL_DIR % num_zone, 'common',  f_name)
        if not isfile(f_filename):
            if debug:
                print "Création du fichier %s" % f_filename
            fic = file(f_filename, 'w')
            fic.write('')
            fic.close()

def make_local_instance_spec_conf_files(num_zone, indice):
    """
        crée les fichiers 'white', 'domain', 'types_mime', 'extensions' dans
        /var/lib/blacklists/dansguardian<zone>/f<indice>/
        et copie les fichiers bannedurllist et bannedsitelist dans ce même repertoire
    """
    dirname = DANS_LOCAL_SUBDIR % (num_zone, indice)
    # les fichiers locaux éditer par l'utilisateur
    for f_name in local_spec_conf_files:
        f_filename = join(dirname, f_name)
        if not isfile(f_filename):
            if debug:
                print "Création du fichier %s" % f_filename
            fic = file(f_filename, 'w')
            fic.write('')
            fic.close()
    # les fichiers locaux dans lesquels on enlève/ajoute des #
    for f_name in local_deported_files:
        src = join(LOCAL_TEMPLATE_DIR, f_name)
        dst = join(dirname, f_name)
        if not isfile(dst):
            if debug:
                print "Création du fichier %s depuis %s" % (dst, src)
            copydata(src, dst)
        else:
            # mise à jour des fichier pour utiliser les même filtres
            # que ceux des templates (#2764)
            lsrc = file(src).readlines()
            ldst = file(dst).readlines()
            final = []
            for line1 in lsrc:
                current = line1
                for line2 in ldst:
                    if line1.endswith(line2):
                        # ligne trouvée avec ou sans # => on l'utilise
                        current = line2
                        break
                final.append(current)
            file(dst, 'w').writelines(final)

def make_local_whitelist_conf_files(num_zone, indice):
    """
        crée le fichier 'whitesitelist', 'whiteurllist' dans
        /var/lib/blacklists/dansguardian<zone>/f<indice>/
    """
    dirname = DANS_LOCAL_SUBDIR % (num_zone, indice)
    # les fichiers locaux éditer par l'utilisateur
    for f_name in local_whitelist_conf_files:
        f_filename = join(dirname, f_name)
        if not isfile(f_filename):
            if debug:
                print "Création du fichier %s" % f_filename
            fic = file(f_filename, 'w')
            fic.write('')
            fic.close()

def create_local_dansguardian_conf(ifaces):
    """ entrée pour la création des fichiers et répertoires
        pour les configurations locales (dans /var/lib/blacklists/)
    """
# pour zone de 1 à n:
#     créer /var/lib/blacklists/<zone>/
#     créer un répertoire commun aux différentes politique d'une zone
#     créer les fichiers locaux communs (spusers, filtergroupslist, hosts, users, urls)
#     pour indice de 1 à num_politique:
#           créer un répertoire /var/lib/blacklists/<zone>/<indice>/ pour les politiques 1,4,5,6,7
#           créer les fichiers locaux spec (domain, white, extensions, typesmime) pour 1,5,6,7
#           créer les fichiers locaux pour la liste blanche (4)

    for num_zone, value in ifaces.items():
        if value != []:
            make_local_instance_dir(num_zone)
            make_local_instance_common_conf_files(num_zone)
            for policy in range(1, num_policies+1):
                if policy not in spec_policies:
                    make_local_instance_subdirs(num_zone, policy)
                    make_local_instance_spec_conf_files(num_zone, policy)
                elif policy in whitelist_policy:
                    make_local_instance_subdirs(num_zone, policy)
                    make_local_whitelist_conf_files(num_zone, policy)

def create_dansguardian_conf_files(ifaces):
    create_main_dansguardian_conf(ifaces)
    create_local_dansguardian_conf(ifaces)

