#-*-coding:utf-8-*-
"""
    gestion des postes et destinations à interdire
    édite les fichiers de configuration en fonction des zones
"""
from amon.backend import get_filter_zones, get_interface
from amon.ipset.time_manager import test_time_range
#import commands
from pyeole.process import system_out
from creole.config import VIRTMASTER, VIRTDISABLED

from os.path import isfile
from creole.parsedico import parse_dico
from amon.era.config import f_poste_name

#IPTABLES_POSTES = """/sbin/iptables -t nat -i %(interface)s -I PREROUTING \
#-s %(poste)s -j REJECT"""
IPTABLES_POSTES = """/sbin/iptables -I FORWARD -i %(interface)s -s %(poste)s \
-j REJECT"""
IPTABLES_POSTES2 = """/sbin/iptables -i %(interface)s -I INPUT -s %(poste)s \
-j REJECT"""

IPTABLES_DEST = """/sbin/iptables -I FORWARD -i %(interface)s -d %(poste)s \
-j REJECT"""
IPTABLES_DEST2 = """/sbin/iptables -I INPUT -i %(interface)s -s %(poste)s -j REJECT"""

IPTABLES_HORAIRE = """/sbin/iptables -I INPUT -p tcp --dport 3128 \
-i %(interface)s -s %(poste)s -m time --timestart %(hdeb)s --timestop \
%(hfin)s --weekdays %%s -j REJECT"""
IPTABLES_HORAIRE2 = """/sbin/iptables -I FORWARD -m multiport -p tcp --dports 80,443,3128,8080 \
-i %(interface)s -o %(gw)s -s %(poste)s -m time --timestart %(hdeb)s \
--timestop %(hfin)s --weekdays %%s -j REJECT"""

CONTAINER_PROXY='proxy'

semaine = ['Mon','Tue','Wed','Thu','Fri','Sat','Sun']
day_translate = {'lundi':'Mon', 'mardi':'Tue', 'mercredi':'Wed',
                'jeudi':'Thu', 'vendredi':'Fri', 'samedi':'Sat',
                'dimanche':'Sun'}

dico = parse_dico()

def get_jliste(jdeb, jfin):
    """
        renvoie les jours compris entre jdeb et jfin
    """
    if jdeb in day_translate.keys():
        jdeb  = day_translate[jdeb]
        jfin = day_translate[jfin]
    istart, iend = semaine.index(jdeb), semaine.index(jfin)
    if istart < iend:
        jliste = ','.join(semaine[istart: iend+1])
    elif istart == iend:
        jliste = semaine[istart]
    else:
        jliste = ','.join(semaine[istart:])
        jliste += ','
        jliste += ','.join(semaine[:iend+1])
    return jliste

############################
# GESTION DES FICHIERS TEXTE
############################
#PARSER:
def parse_all(line):
    """ parse une ligne du fichier d'interdiction totale """
    line = line.strip()
    if len(line.split('#')) == 2:
        poste, interface = line.split('#')
        return {'interface':interface,
                'poste':poste}
    return None

def parse_web(line):
    """ parse une ligne du fichier d'interdiction web """
    line = line.strip()
    if len(line.split('#')) == 6:
        poste,  hdeb, hfin, jdeb, jfin, interface = line.split('#')
        return dict(poste=poste,
                    hdeb=hdeb,
                    hfin=hfin,
                    jdeb=jdeb,
                    jfin=jfin,
                    interface=interface)
    elif len(line.split('#')) == 5:
        poste,  hdeb, hfin, jdeb, jfin = line.split('#')
        return dict(poste=poste,
                    hdeb=hdeb,
                    hfin=hfin,
                    jdeb=jdeb,
                    jfin=jfin)
    return None

poste_syntax = {'web': "%(poste)s#%(hdeb)s#%(hfin)s#%(jdeb)s#%(jfin)s#%(interface)s",
                'all': "%(poste)s#%(interface)s",
                'destination':"%(poste)s#%(interface)s",
               }
poste_keys = {'web':('poste',  'hdeb', 'hfin', 'jdeb', 'jfin'),
              'all':('poste',),
              'destination':('poste',)}

poste_inverse_syntax = {'all':parse_all,
                        'web':parse_web,
                        'destination':parse_all}


def load_forbidden_poste(zone, mode):
    """
        récupère la  liste des poste interdits
        @zone: indice de la configuration
        @mode: mode d'interdiction
    """
    f_name = f_poste_name[mode]%zone
    if isfile(f_name):
        return [poste_inverse_syntax[mode](line)
                for line in file(f_name).read().splitlines()]
    else:
        return []

def save_forbidden_poste(zone, mode, postes):
    """
        sauvegarde une liste de postes interdits
        @zone: indice de la configuration
        @mode: mode d'interdiction
        @postes: liste des postes à interdire
            si l'interface n'est pas précisée on tente
            de la récupérer depuis l'ip du poste
            sinon on renvoie le premier eth de la zone
    """
    if type(postes) != list:
        postes = [postes]
    f_name = f_poste_name[mode]%zone
    to_w = ''
    for poste in postes:
        if False in [i in poste.keys() for i in poste_keys[mode]]:
            raise Exception, "Il manque des données pour les interdictions de poste."
        if not poste.has_key('interface'):
            poste['interface'] = get_interface(zone, poste['poste'])
        to_w += '%s\n'%poste_syntax[mode]%poste

    if mode == 'web':
        for poste in postes:
            if not test_time_range(poste['hdeb'], poste['hfin']):
                raise Exception, "Erreur: l'heure de début doit précéder l'heure de fin."

    f_postes = file(f_name, 'w')
    f_postes.write(to_w)
    f_postes.close()
    return True


#######################
# GENERATEURS DE REGLES
#######################

def interdit_all_postes(zone, poste, interface=None):
    """
        interdit le poste 'poste' de réseau sur la zone (politique de filtrage)
        d'indice 'zone', on écrit la règle pour l'interface où le poste
        est connecté et on détecte automatiquement l'interface
        dans le cas où elle n'est pas renseignée
    """
    if interface is None and not poste.has_key('interface'):
        poste['interface'] = get_interface(zone, poste['poste'])
    ret = [(IPTABLES_POSTES%(poste), VIRTMASTER), (IPTABLES_POSTES2%(poste), VIRTMASTER)]
    if not VIRTDISABLED:
        ret.append((IPTABLES_POSTES2%(poste), CONTAINER_PROXY))
    return ret

def interdit_destination(zone, poste):
    """
        interdit le poste 'poste' en tant que destination
        @zone: indice de la configuration à éditer
        @poste: identifiant du poste (sous-réseau)
        on écrit la règle pour chaque interface de la zone
    """
    regles = []
    if not poste.has_key('interface'):
        for interface in get_interface(zone):
            poste['interface'] = interface
            regles.append((IPTABLES_DEST % (poste), VIRTMASTER))
            regles.append((IPTABLES_DEST2 % (poste), VIRTMASTER))
    else:
        regles.append((IPTABLES_DEST % (poste), VIRTMASTER))
        regles.append((IPTABLES_DEST2 % (poste), VIRTMASTER))
    return regles

def interdit_postes(zone, poste, interface=None):
    """
        interdit un poste de navigation internet selon des horaires*
        @zone: indice de la configuration à éditer
        @poste: de la forme {'poste':'ip du poste',
                             'hdeb': debut de plage,
                             'hfin': fin de plage,
                             'gw': gateway (interface de sortie vers le web),
                             'jdeb': jour de debut
                             'jfin': jour de fin
                             }
    """
    if interface is None and not poste.has_key('interface'):
        poste['interface'] = get_interface(zone, poste['poste'])
    poste['gw'] = dico["interface_gw"]
    jliste = get_jliste(poste['jdeb'], poste['jfin'])
    ret = [(IPTABLES_HORAIRE%poste%jliste, CONTAINER_PROXY),
            (IPTABLES_HORAIRE2%poste%jliste, VIRTMASTER)]
    return ret

def generate_postes_rules():
    """
        génère les règles pour les interdictions de poste
        web
    """
    regles = []
    for zone in get_filter_zones().keys():
        if isfile(f_poste_name['web']%zone):
            for poste in load_forbidden_poste(zone, 'web'):
                regles.extend(interdit_postes(zone, poste))
    return regles

def generate_all_postes_rules():
    """
        génère les règles pour l'interdiction totale de poste
    """
    regles = []
    for zone in get_filter_zones().keys():
        if isfile(f_poste_name['all']%zone):
            for poste in load_forbidden_poste(zone, 'all'):
                regles.extend(interdit_all_postes(zone, poste))
    return regles

def generate_destinations_rules():
    """
        génère les règles pour les interdictions de destinations
    """
    regles = []
    for zone in get_filter_zones().keys():
        if isfile(f_poste_name['destination']%zone):
            for poste in load_forbidden_poste(zone, 'destination'):
                regles.extend(interdit_destination(zone, poste))
    return regles

def generate_rules():
    """
       génère les règles iptables pour l'interdiction de postes
    """
    if not VIRTDISABLED:
        flush_iptables = [['/sbin/iptables', '-t', 'filter', '-F'],
        ['/sbin/iptables', '-t', 'mangle', '-F'],
        ['/sbin/iptables', '-t', 'nat', '-F'],
        ['/sbin/iptables', '-t', 'filter', '-X'],
        ['/sbin/iptables', '-t', 'mangle', '-X'],
        ['/sbin/iptables', '-t', 'nat', '-X'],
        ['/sbin/iptables', '-P', 'INPUT', 'ACCEPT'],
        ['/sbin/iptables', '-P', 'OUTPUT', 'ACCEPT'],
        ['/sbin/iptables', '-P', 'FORWARD', 'DROP']]
        for rule in flush_iptables:
            ret, out, err = system_out(rule, container=CONTAINER_PROXY)
            if ret != 0:
                raise Exception, "Erreur à la mise en place des restrictions de postes/destinations : %s, %s"%(out, err)

    regles = generate_postes_rules()
    regles.extend(generate_all_postes_rules())
    regles.extend(generate_destinations_rules())
    for rule, container in regles:
        rule = rule.strip().split()
        ret, out, err = system_out(rule, container=container)
        if ret != 0:
            raise Exception, "Erreur à la mise en place des restrictions de postes/destinations : %s, %s"%(out, err)
    return True

