# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Génère les règles iptables pour les groupes de machine
#
###########################################################################
from twisted.python import log

from amon.ipset import group_manager
from amon.ipset import time_manager
from amon.backend import get_filter_zones
from pyeole.process import system_out
from creole.config import VIRTMASTER, VIRTDISABLED

#Pour avoir des prints
debug = False

#### template pour les règles iptables
# partie ipset de la règle avec gestion de ports (portset webservices)
GROUP_RULE = """-i %(interface)s -m set --set %(name)s src,dst"""
# sans la gestion de ports
TOTAL_GROUP_RULE = """-i %(interface)s -m set --set %(name)s src"""

# règles Web :
WEB_TIME_INPUT_RULE =  """/sbin/iptables -I INPUT -p tcp %s %s -j REJECT"""
WEB_TIME_FORWARD_RULE = """/sbin/iptables -o %s -I FORWARD -m multiport -p tcp --dports 80,443,3128,8080 %s %s -j REJECT"""

WEB_INPUT_RULE =  """/sbin/iptables -I INPUT -p tcp %s -j REJECT"""
WEB_FORWARD_RULE = """/sbin/iptables -o %s -I FORWARD -m multiport -p tcp --dports 80,443,3128,8080 %s -j REJECT"""

# règles horaires :
TIME_RULE = """-m time --timestart %(hdeb)s --timestop %(hfin)s --weekdays %(jour)s"""

# règle totale
TOTAL_INPUT_RULE = """/sbin/iptables -I INPUT %s -j REJECT"""
TOTAL_PREROUTING_RULE = """/sbin/iptables -I FORWARD %s -j REJECT"""

# Traduction des jours
JOUR = dict(lundi='Mon', mardi='Tue', mercredi='Wed', jeudi='Thu',
            vendredi='Fri', samedi='Sat', dimanche='Sun')

# Liste des jours
semaine = ['Mon','Tue','Wed','Thu','Fri','Sat','Sun']

class IptablesGenerator:
    """
        génère les règles iptables pour les set d'ips en fonction des interdictions
        1 - creation des regles pour le module ipset restreint au port 3128 et sans restriction
        2 - séparation des règles web ou total réseau
        3 - traitement horaires ou pas horaire
    """

    def launch_rule(self, rule, container=VIRTMASTER):
        """
            lance l'éxécution d'une règle iptable
        """
        #desactive les règles dans le conteneur si désactivé
        if container != VIRTMASTER and VIRTDISABLED:
            return
        code, out, err = system_out(rule.split(), container=container)
        if debug:
            print("Execution de %s depuis iptables_generator.py"%(rule))
            print code, out, err


    def create_rules(self):
        """
        Génération des règles iptables pour les groupes de machines
        """
        from creole.parsedico import parse_dico
        I_GW = parse_dico()["interface_gw"]
        for zone, active_groups in get_active_groups().items():
            for group in active_groups:
                group_datas = get_group(zone, group)

                total_group_rule = TOTAL_GROUP_RULE % group_datas
                group_rule = GROUP_RULE % group_datas
                ## si web
                ##      creer regles generales
                ##      si horaires:
                ##          generation des regles horaires
                ##      sinon
                ##          generation des regles sans horaires
                ## sinon
                ##      generation des regles globales

                if group_datas['active'] != '2':                    # interdiction web
                    if group_datas.has_key('schedules'):            # avec horaire
                        for schedule in group_datas['schedules']:
                            time_rule = TIME_RULE % schedule

                            rule = WEB_TIME_INPUT_RULE % (group_rule, time_rule)
                            self.launch_rule(rule)
                            self.launch_rule(rule, 'proxy')

                            rule = WEB_TIME_FORWARD_RULE % (I_GW, total_group_rule, time_rule)
                            self.launch_rule(rule)
                    else:                                           # sans horaire (tout le temps)
                        rule = WEB_INPUT_RULE % (group_rule)
                        self.launch_rule(rule)
                        self.launch_rule(rule, 'proxy')

                        rule = WEB_FORWARD_RULE % (I_GW, total_group_rule)
                        self.launch_rule(rule)

                else: ## interdiction totale
                    rule = TOTAL_INPUT_RULE % total_group_rule
                    self.launch_rule(rule)
                    self.launch_rule(rule, 'proxy')

                    rule = TOTAL_PREROUTING_RULE % total_group_rule
                    self.launch_rule(rule)

# fonctions utiles
def get_active_groups():
    """ charger la liste des groupes actifs """
    return dict([(zone,
                 [group for group in group_manager.get_all_groups(zone) if group['active'] != '0'])
                 for zone in get_filter_zones().keys()])

def get_group(zone, group):
    """ renvoie le groupe group formaté pour la mise en place des règles iptables
        @group: dico de description du groupe
    """
    group_schedules = time_manager.get_schedules(zone, group['name'])
    if group_schedules != {}:
        if group['time'] == '1':
            group['schedules'] = format_schedules(group_schedules)
    return group

def format_schedules(schedules):
    """ formatte les plages horaires afin de minimiser le nombre de règle iptables à implémenter
        on interdit tout sauf les horaires indiqués
    """
    def compare_schedules(a,b):
        """ compare deux plages horaires """
        a = a['hdeb']
        b = b['hdeb']
        if int(a.split(':')[0])>int(b.split(':')[0]):
            return 1
        elif int(a.split(':')[0]) == int(b.split(':')[0]):
            return 0
        else:
            return -1

    formatted_schedules = []
    for jour in JOUR.keys():
        horaires = schedules.get(jour, [])
        deb = '0:00'
        horaires.sort(compare_schedules)
        for horaire in horaires:
            # si le debut de plage n'est pas égal à la fin (pour le début de journée de 0:00 à 0:00)
            if horaire['hdeb'] != deb:
                formatted_schedules.append({'jour':JOUR[jour], 'hdeb':deb, 'hfin': horaire['hdeb']})
            deb = horaire['hfin']
        # si le début de la nouvelle plage ne correspond pas à la fin de la journée
        if deb not in ['23:59', '24:00']:
            formatted_schedules.append({'jour':JOUR[jour], 'hdeb':deb, 'hfin':'23:59'})
    return formatted_schedules


def get_jliste(jdeb, jfin):
    istart, iend = semaine.index(jour[jdeb]), semaine.index(jour[jfin])
    if istart < iend:
        jliste = ','.join(semaine[istart: iend+1])
    elif istart == iend:
        jliste = semaine[istart]
    else:
        jliste = ','.join(semaine[istart:])
        jliste += ','
        jliste += ','.join(semaine[:iend+1])
    return jliste

